/* Autogenerated by png2wx.pl on Fri Sep  9 13:06:36 2011 */
#ifndef MY_ICONS_H
#define MY_ICONS_H 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#    include <wx/wx.h>
#endif

extern void initialize_my_images(void);
extern void cleanup_my_images(void);

extern wxBitmap *_img_radar;
extern wxBitmap *_img_aisradar_pi;

#endif /* MY_ICONS_H */
