/* Autogenerated by png2wx.pl on Tue Jul  8 18:17:53 2014 */
#ifndef TARGET_H
#define TARGET_H 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#	include <wx/wx.h>
#endif

extern void initialize_images(void);

extern wxBitmap *_img_target;

#endif /* TARGET_H */
