/* Autogenerated by png2wx.pl on Tue Jul  8 18:20:04 2014 */
#ifndef YACHT_UNK_H
#define YACHT_UNK_H 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#	include <wx/wx.h>
#endif

extern void initialize_images(void);

extern wxBitmap *_img_yacht_unk;

#endif /* YACHT_UNK_H */
