/* Autogenerated by png2wx.pl on Tue Jul  8 18:19:01 2014 */
#ifndef TARGET_LOST_H
#define TARGET_LOST_H 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#	include <wx/wx.h>
#endif

extern void initialize_images(void);

extern wxBitmap *_img_target_lost;

#endif /* TARGET_LOST_H */
