/* Autogenerated by png2wx.pl on Mon Sep 19 17:33:44 2011 */
#ifndef ICON_H
#define ICON_H 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#	include <wx/wx.h>
#endif

extern void initialize_images(void);

extern wxBitmap *_img_base;

#endif /* ICON_H */
