/* Autogenerated by png2wx.pl on Tue Jul  8 18:18:16 2014 */
#ifndef TARGET_UNK_H
#define TARGET_UNK_H 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#	include <wx/wx.h>
#endif

extern void initialize_images(void);

extern wxBitmap *_img_target_unk;

#endif /* TARGET_UNK_H */
